#include "dpmsscreen.h"

dpmsscreen::dpmsscreen(void)
 : BWindowScreen("DPMS screen", B_8_BIT_640x480)
{
	blanking_mode = 0;
	blanked = 0;
	Show();
}

void dpmsscreen::ScreenConnected(bool active)
{
	if(active)
	{
		vga = (unsigned char *)IOBase();
		blank(2);
	}
	else
		unblank();
}

#define video_port_reg	(0x3d4)
#define video_port_val	(0x3d5)
#define seq_port_reg	(0x3c4)		/* Sequencer register select port */
#define seq_port_val	(0x3c5)		/* Sequencer register value port  */
#define video_misc_rd	(0x3cc)		/* Video misc. read port	  */
#define video_misc_wr	(0x3c2)		/* Video misc. write port	  */

/* routine to blank a vesa screen */
void dpmsscreen::blank(short mode)
{
	/* save original values of VGA controller registers */
	SeqCtrlIndex = vga[seq_port_reg];
	CrtCtrlIndex = vga[video_port_reg];
	CrtMiscIO = vga[video_misc_rd];

	if(mode == 2)
	{ 
	    vga[video_port_reg] = 0x00;			/* HorizontalTotal */
	    HorizontalTotal = vga[video_port_val];
	    vga[video_port_reg] = 0x01;			/* HorizDisplayEnd */
	    HorizDisplayEnd = vga[video_port_val];
	    vga[video_port_reg] = 0x04;			/* StartHorizRetrace */
	    StartHorizRetrace = vga[video_port_val];
	    vga[video_port_reg] = 0x05;			/* EndHorizRetrace */
	    EndHorizRetrace = vga[video_port_val];
	}
	vga[video_port_reg] = 0x07;					/* Overflow */
	Overflow = vga[video_port_val];
	vga[video_port_reg] = 0x10;					/* StartVertRetrace */
	StartVertRetrace = vga[video_port_val];
	vga[video_port_reg] = 0x011;				/* EndVertRetrace */
	EndVertRetrace = vga[video_port_val];
	vga[video_port_reg] = 0x17;					/* ModeControl */
	ModeControl = vga[video_port_val];
	vga[seq_port_reg] = 0x01;					/* ClockingMode */
	ClockingMode = vga[seq_port_val];
	
	/* assure that video is enabled */
	/* "0x20" is VIDEO_ENABLE_bit in register 01 of sequencer */
	vga[seq_port_reg] = 0x01;
	vga[seq_port_val] = ClockingMode | 0x20;

	/* test for vertical retrace in process.... */
	if((CrtMiscIO & 0x80) == 0x80)
		vga[video_misc_wr] = CrtMiscIO & 0xef;

	/* Set <End of vertical retrace> to minimum (0) and
	 * <Start of vertical Retrace> to maximum (incl. overflow)
	 * Result: turn off vertical sync (VSync) pulse.
	 */
	vga[video_port_reg] = 0x10;		/* StartVertRetrace */
	vga[video_port_val] = 0xff;		/* maximum value */
	vga[video_port_reg] = 0x11;		/* EndVertRetrace */
	vga[video_port_val] = 0x40;		/* minimum (bits 0..3)  */
	vga[video_port_reg] = 0x07;		/* Overflow */
	vga[video_port_val] = Overflow | 0x84;	/* bits 9,10 of  */
	/* vert. retrace */
      
	if(mode == 2)
	{
	    /* Set <End of horizontal retrace> to minimum (0) and
	     *  <Start of horizontal Retrace> to maximum
	     * Result: turn off horizontal sync (HSync) pulse.
	     */
	    vga[video_port_reg] = 0x04;	/* StartHorizRetrace */
	    vga[video_port_val] = 0xff;	/* maximum */
	    vga[video_port_reg] = 0x05;	/* EndHorizRetrace */
	    vga[video_port_val] = 0x00;	/* minimum (0) */
	}

	/* restore both index registers */
	vga[seq_port_reg] = SeqCtrlIndex;
	vga[video_port_reg] = CrtCtrlIndex;

	blanked = mode;
}	

/* routine to unblank a vesa screen */
void dpmsscreen::unblank(void)
{
	if(! blanked)
		return;

	/* restore original values of VGA controller registers */
	vga[video_misc_wr] = CrtMiscIO;

	if(blanked == 2)
	{
	    vga[video_port_reg] = 0x00;	/* HorizontalTotal */
	    vga[video_port_val] = HorizontalTotal;
	    vga[video_port_reg] = 0x01;	/* HorizDisplayEnd */
	    vga[video_port_val] = HorizDisplayEnd;
	    vga[video_port_reg] = 0x04;	/* StartHorizRetrace */
	    vga[video_port_val] = StartHorizRetrace;
	    vga[video_port_reg] = 0x05;	/* EndHorizRetrace */
	    vga[video_port_val] = EndHorizRetrace;
	}
	vga[video_port_reg] = 0x07;			/* Overflow */
	vga[video_port_val] = Overflow;
	vga[video_port_reg] = 0x10;			/* StartVertRetrace */
	vga[video_port_val] = StartVertRetrace;
	vga[video_port_reg] = 0x11;			/* EndVertRetrace */
	vga[video_port_val] = EndVertRetrace;
	vga[video_port_reg] = 0x17;			/* ModeControl */
	vga[video_port_val] = ModeControl;
	vga[seq_port_reg] = 0x01;			/* ClockingMode */
	vga[seq_port_val] = ClockingMode;

	/* restore index/control registers */
	vga[seq_port_reg] = SeqCtrlIndex;
	vga[video_port_reg] = CrtCtrlIndex;

	blanked = 0;
}
